package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Root;
import java.time.LocalDate;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            findEmployeeByName();
            findEmployeesByDept();
            findEmployeeBySalary();
            findEmployeeByJoinDate();
            findEmployeeWithNullJoinDate();

        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", 2000, Dept.IT,
                LocalDate.of(2010, 4, 1));
        Employee employee2 = Employee.create("Rose", 4500, Dept.Admin,
                LocalDate.of(2009, 7, 1));
        Employee employee3 = Employee.create("Denise", 2000, Dept.Admin,
                LocalDate.of(2002, 2, 1));
        Employee employee4 = Employee.create("Mike", 4000, Dept.IT,
                LocalDate.of(2007, 10, 1));
        Employee employee5 = Employee.create("Linda", 4500, Dept.Sales,
                null);
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.persist(employee5);
        em.getTransaction().commit();
        System.out.println("-- employee persisted --");
        CriteriaQuery<Employee> query = em.getCriteriaBuilder()
                                          .createQuery(Employee.class);
        query.select(query.from(Employee.class));
        em.createQuery(query).getResultList().forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeByName() {
        System.out.println("-- Finding employee by name --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cBuilder.createQuery(Employee.class);
        Root<Employee> employee = query.from(Employee.class);
        Expression<String> linda = cBuilder.literal("Linda");
        query.select(employee)
             .where(cBuilder.equal(employee.get(Employee_.name), linda));
        TypedQuery<Employee> tq = em.createQuery(query);
        List<Employee> resultList = tq.getResultList();
        resultList.forEach(System.out::println);
    }

    private static void findEmployeesByDept() {
        System.out.println("-- Finding employee by dept IT --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cBuilder.createQuery(Employee.class);
        Root<Employee> employee = query.from(Employee.class);
        query.select(employee)
             .where(cBuilder.equal(employee.get(Employee_.DEPT), Dept.IT));
        TypedQuery<Employee> tq = em.createQuery(query);
        List<Employee> resultList = tq.getResultList();
        resultList.forEach(System.out::println);
    }

    private static void findEmployeeBySalary() {
        System.out.println("-- finding employee with salary greater than 4000 inclusively --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cBuilder.createQuery(Employee.class);
        Root<Employee> employee = query.from(Employee.class);
        query.select(employee)
             .where(cBuilder.ge(employee.get(Employee_.salary), cBuilder.literal(4000)));
        TypedQuery<Employee> tq = em.createQuery(query);
        List<Employee> resultList = tq.getResultList();
        resultList.forEach(System.out::println);
    }

    private static void findEmployeeByJoinDate() {
        System.out.println("-- finding employee with join date between 2005 to 2010 inclusively --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cBuilder
                .createQuery(Employee.class);
        Root<Employee> employee = query.from(Employee.class);
        query.select(employee)
             .where(cBuilder.between(employee.get(Employee_.JOIN_DATE),
                     LocalDate.of(2005, 1, 1),
                     LocalDate.of(2011, 1, 1)
             ));
        TypedQuery<Employee> tq = em.createQuery(query);
        List<Employee> resultList = tq.getResultList();
        resultList.forEach(System.out::println);
    }

    private static void findEmployeeWithNullJoinDate() {
        System.out.println("-- finding employee with null join date  --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cBuilder.createQuery(Employee.class);
        Root<Employee> employee = query.from(Employee.class);
        query.select(employee)
             .where(cBuilder.equal(employee.get(Employee_.JOIN_DATE),
                cBuilder.nullLiteral(LocalDate.class)));
        TypedQuery<Employee> tq = em.createQuery(query);
        List<Employee> resultList = tq.getResultList();
        resultList.forEach(System.out::println);
    }
}